
## install_packages.py - Language Support Installtion using PackageKit
##
## Copyright (C) 2013-2014 Red Hat, Inc.
## Authors: Parag Nemade <pnemade@redhat.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import yum
import subprocess
import os
from gi.repository import Gio, GLib

import gettext
_ = lambda x: gettext.ldgettext("system-config-language", x)
N_ = lambda x: x

uniq_installed_pkgs = []
conffile = None
lang_pkgs = []
installed_list = []
list_to_install = []
langlist_with_cc = ['zh_CN', 'zh_TW', 'pt_BR']
conditional_pkgs = {}
pkgmatches = []

def installed_packages():
    """ Get the installed package list on the system """

    global conffile
    yb = yum.YumBase()
    conffile = yb.conf.persistdir + '/plugins/langpacks/installed_langpacks'

    installed_pkgs = yb.rpmdb.returnPackages()
    for pkg in sorted(set(installed_pkgs)):
        if pkg.name not in uniq_installed_pkgs:
            uniq_installed_pkgs.append(pkg.name)

def write_installed_langpacks(langplist):
    """ Write the installed langpacks from conffile """

    global conffile
    if not conffile:
        return
    confdir = os.path.dirname(conffile)
    if not os.path.exists(confdir):
        try:
            os.makedirs(confdir, 0755)
        except (IOError, OSError), e:
            return
    try:
        tmp = open(conffile + ".tmp", "w+")
        for line in langplist:
            tmp.write(line + "\n")
        tmp.close()
        os.rename(conffile + ".tmp", conffile)
    except (IOError, OSError), e:
        return

def read_installed_langpacks():
    """ Read the installed langpacks from conffile """

    global conffile
    if not conffile:
        return []
    ret = []
    try:
        f = open(conffile, "r")
        flist = f.readlines()
        f.close()
    except (IOError, OSError), e:
        return []
    for item in flist:
        item = item.strip()
        ret.append(item)
    return ret

def add_langpack_to_installed_list(lang):
    """ Add the language code to the installed languages file """
    modified = 0
    langplist = read_installed_langpacks()
    if lang not in langplist:
        langplist.append(lang)
        modified = 1
    if modified:
        write_installed_langpacks(langplist)

def check_if_already_installed(check_lang):
    """ Check if the requested langpack is already installed on the system """
    langpack_list = read_installed_langpacks()
    if check_lang not in langpack_list:
        print("{0} language not installed").format(check_lang)
        return "no"
    else:
        print("{0} language is already installed").format(check_lang)
        return "yes"

def get_language_packages(lang):
    """ Find the package list required to install the requested langpack """
    cmd = "yum langinfo {0}".format(lang)
    for line in subprocess.check_output(cmd, shell=True).split('\n'):
        if line.startswith("  "):
            lang_pkgs.append(line[2:])
    print "Langpacks provides packages => {0}".format(lang_pkgs)

    return lang_pkgs

def _setup_conditional_pkgs(repos):
    """ This is called to check if cond pkg already setup """
    # This takes ~0.2 seconds to init, so only do it if we need to
    if not conditional_pkgs:
        my_postreposetup_hook(repos)

def iterparse(filename):
    try:
        for e in yum.misc.cElementTree_iterparse(filename):
            yield e
    except SyntaxError, e:
        print >>sys.stderr, '%s: %s' % (filename, str(e))

# This takes ~0.2 seconds to init, so don't do it for non-transaction commands.
# This does mean we might end up downloading the groups file in postresolve,
# but meh.
def my_postreposetup_hook(repos):
    global conditional_pkgs

    for repo in repos:
        try:
            infile = repo.getGroups()
        except yum.Errors.YumBaseError:
            continue

        if not infile:
            continue
        infile = yum.misc.decompress(infile, check_timestamps=True)
        for event, elem in iterparse(infile):
            if elem.tag == "langpacks":
                for child in elem.getchildren():
                    if child.tag != "match":
                        continue
                    name = child.get("name")
                    install = child.get("install")

                    if name not in conditional_pkgs:
                        conditional_pkgs[name] = []
                    conditional_pkgs[name].append(install)


def get_matches(sack, list):
    ret = []
    for match in list:
        p = sack.searchProvides(match)
        if p:
            ret.append(match)
    return ret

def pk_install(langcode, list_to_install):
    """ PK API to install required language support packages """
    bus_type = Gio.BusType.SESSION
    flags = 0
    iface_info = None

    # Creating proxies does not do any blocking I/O, and never fails
    proxy = Gio.DBusProxy.new_for_bus_sync(
            bus_type,
            flags,
            iface_info,
            'org.freedesktop.PackageKit',
            '/org/freedesktop/PackageKit',
            'org.freedesktop.PackageKit.Modify',
             None)
    try:
        proxy.set_default_timeout(0x7fffffff) # timeout in milliseconds
        proxy.InstallPackageNames("(uass)", 0, list_to_install,
                                   "show-confirm-search,hide-finished")
        add_langpack_to_installed_list(langcode)
        return "LangSupportInstalled"
    except GLib.GError as e:
        #When you click on cancel
        print "Unable to use PackageKit: {0}".format(e)
        return "LangPkgInstallCanceled"

class InstallLangpack:
    """ Class for installing packages using PackageKit """
    def __init__(self):
        """ init method """
        installed_packages()

    @staticmethod
    def install_packages(lang):
        """ Find which packages are required for installation """
        yb = yum.YumBase()
        yb.repos.doSetup()
        _setup_conditional_pkgs(yb.repos.listEnabled())
        sack = yb.pkgSack

        if lang not in langlist_with_cc:
            langcode = lang.split("_")[0]
        else:
            langcode = lang

        installed = check_if_already_installed(langcode)

        if installed is "no":
            all_pkg_list = get_language_packages(langcode)
            if all_pkg_list:
                for basepkg in conditional_pkgs:
                    if yb.rpmdb.searchNevra(name=basepkg) or yb.tsInfo.matchNaevr(name=basepkg):
                        conds = conditional_pkgs[basepkg]
                        patterns = map(lambda x: x % (lang,), conds)
                        shortlang = lang.split('_')[0]
                        if shortlang != lang:
                            patterns = patterns + map(lambda x: x % (shortlang,), conds)
                        for p in patterns:
                            if p not in pkgmatches:
                                pkgmatches.append(p)

                all_match_pkg_list = get_matches(sack, pkgmatches)

                for pkg in all_match_pkg_list:
                    if pkg not in uniq_installed_pkgs:
                        list_to_install.append(pkg)
                list_to_install.sort()
                print "Packages to be installed=> {0}".format(list_to_install)

                if not list_to_install:
                    print "Langpack already installed"
                    add_langpack_to_installed_list(langcode)
                    return "LangPkgAlreadyInstalled"
                else:
                    ret = pk_install(langcode, list_to_install)
                    return ret
            else:
                print "We got no packages for requested language"
                return "LangPkgNoSupport"
        else:
            return "LangPackAlreadyInstalled"
